-- Test payment_account
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(38);

set search_path to numerus, auth, public;

select has_table('payment_account');
select has_pk('payment_account');
select table_privs_are('payment_account', 'guest', array []::text[]);
select table_privs_are('payment_account', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_account', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_account', 'authenticator', array []::text[]);


select has_column('payment_account', 'payment_account_id');
select col_is_pk('payment_account', 'payment_account_id');
select col_type_is('payment_account', 'payment_account_id', 'integer');
select col_not_null('payment_account', 'payment_account_id');
select col_hasnt_default('payment_account', 'payment_account_id');

select has_column('payment_account', 'company_id');
select col_is_fk('payment_account', 'company_id');
select fk_ok('payment_account', 'company_id', 'company', 'company_id');
select col_type_is('payment_account', 'company_id', 'integer');
select col_not_null('payment_account', 'company_id');
select col_hasnt_default('payment_account', 'company_id');

select has_column('payment_account', 'slug');
select col_is_unique('payment_account', 'slug');
select col_type_is('payment_account', 'slug', 'uuid');
select col_not_null('payment_account', 'slug');
select col_has_default('payment_account', 'slug');
select col_default_is('payment_account', 'slug', 'gen_random_uuid()');

select has_column('payment_account', 'payment_account_type');
select col_type_is('payment_account', 'payment_account_type', 'text');
select col_is_fk('payment_account', 'payment_account_type');
select fk_ok('payment_account', 'payment_account_type', 'payment_account_type', 'payment_account_type');
select col_not_null('payment_account', 'payment_account_type');
select col_hasnt_default('payment_account', 'payment_account_type');

select has_column('payment_account', 'name');
select col_type_is('payment_account', 'name', 'text');
select col_not_null('payment_account', 'name');
select col_hasnt_default('payment_account', 'name');


set client_min_messages to warning;
truncate payment_account cascade;
truncate company_user cascade;
truncate company cascade;
truncate payment_method cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into payment_account (company_id, payment_account_type, name)
values (2, 'bank', 'Bank A')
     , (4, 'cash', 'Cash')
;


prepare payment_account_data as
select company_id, name
from payment_account
order by company_id, name;

set role invoicer;
select is_empty('payment_account_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'payment_account_data',
	$$ values (2, 'Bank A')
	$$,
	'Should only list payment_accounts from the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'payment_account_data',
	$$ values (4, 'Cash')
	$$,
	'Should only list payment_accounts from the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'payment_account_data',
	'42501', 'permission denied for table payment_account',
	'Should not allow select to guest users'
);
reset role;

select throws_ok( $$
	insert into payment_account (company_id, payment_account_type, name)
	values (2, 'cash', '  ')
	$$,
       '23514', 'new row for relation "payment_account" violates check constraint "payment_account_name_not_empty"',
	'Should not allow payment accounts with a blank name'
);


select *
from finish();

rollback;

